#!/opt/VRTSsfmh/bin/perl -I/opt/VRTSsfmh/lib/modules

BEGIN { @INC = ('/opt/VRTSsfmh/lib/modules') if ( $^O !~ /Win32/i ); }
use strict;
require VRTS::Paths;
use VRTS::Logger;
use JSON;
use Data::Dumper;

my $logdir = VRTS::Paths::get_path('LogDir');
my $log = new VRTS::Logger(
    Module    => 'add_vrp_role_to_nbu',
    Logdir    => $logdir,
    Level     => 'debug',
    Logfile   => 'add_vrp_role_to_nbu.log',
    Logsize   => 1024000,
    UmiString => 'V-0000-0000'
);

# List of all permission required by VRP
my @vrp_permission = (
	'View Recovery Points',
	'Recover/Restore',
	'Instant Access',
	'Download Files',
	'Restore Files',
	'Manage Hosts',
	'Manage Policies',
	'Manage SLPs',
	'Manage Assets',
	'Manage Application Servers and Asset Groups',
	'Manage Appservers and Asset Groups',
	'Manage NetBackup',
	'View Storage',
	'Overwrite Asset',
	'View Hosts'
);

# Map to store name to id mapping for creating role
my %nbu_permission_map = {};

my ($master, $user, $password, $cacert_file, $jwt, $CURL_GET, $CURL_POST, $out_json , $version);

sub exec_cmd {
	my ($cmd, $do_not_log) = @_;
	if (!defined $do_not_log || $do_not_log != 1) {
		VRTS::Logger::log('debug', "Command :[$cmd]");
	}
	my $cmdout = `$cmd`;
	VRTS::Logger::log('debug', "Command output:[$cmdout]");
	if ($cmdout eq "") {
		return $cmdout;
	}
	eval { $out_json = JSON::from_json($cmdout);} or do {
        	printf ("\nError: Failed to parse the json output: $cmdout, $@\n");
	        printf ("\nError: Failed to parse due to error : [$@]\n");
        	VRTS::Logger::log('error', "Failed to parse the json output: $cmdout, due to error [$@]");
	};
	if (exists ($out_json->{'errorCode'})) {
        	printf("\nError: API failed with error: $out_json->{'errorMessage'}\n");
        	VRTS::Logger::log('error', "API failed with error: ".$out_json->{'errorMessage'});
        	VRTS::Logger::log('error', "API failed with error: ".$out_json->{'errorCode'});
        	exit(1);
	}
	return $out_json; 
}

sub get_inputs
{
        print "Enter fully qualified host name of the NBU Master: ";
        $master = <STDIN>;
        chomp $master;

        print "Enter NBU Master version(8.1.2/8.2): ";
        $version = <STDIN>;
        chomp $version;

        print "Enter username: ";
        $user = <STDIN>;
        chomp $user;

        print "Enter password: ";
        system('stty','-echo');
        chop($password=<STDIN>);
        system('stty','echo');
	VRTS::Logger::log('debug', "Script called with argument nbumaster name[$master], user [$user], version [$version]");
}

sub get_cacert
{
        my $cmd = "curl -s https://$master:1556/netbackup/security/cacert -k";
        my $cert_json = exec_cmd($cmd);
	if (!defined $cert_json || $cert_json eq "") {
		printf("\nError: Failed to fetch certificate\n");
		VRTS::Logger::log('error',"Erro: Failed to fetch certificate");
		exit(1);
	}
        exists($cert_json->{'webRootCert'}) || die "No webRootCert received.\nExiting.\n";
        $cacert_file = "/tmp/$master"."-cacert.pem";
	my $fh;
        open($fh, ">", $cacert_file) || die "Cannot open file $cacert_file in writing mode\n";
        print $fh $cert_json->{'webRootCert'};
        close $fh;
        VRTS::Logger::log("debug", "cacert stored in $cacert_file");
}

sub login
{
        my $cmd = "curl -s -X POST https://$master:1556/netbackup/login -H 'content-type: application/vnd.netbackup+json;version=1.0' -d '{\"userName\":\"$user\",\"password\":\"$password\"}' --cacert $cacert_file";
        my $login_json = exec_cmd($cmd);
	if (!defined $login_json || $login_json eq "") {
		printf("\nError: Failed to get token\n");
		VRTS::Logger::log('error',"Erro: Failed to get token");
		exit(1);
	}
        exists($login_json->{'token'}) || die "\nJWT not received in login API. Exiting\n";
        VRTS::Logger::log("debug", "Login successful with output $login_json");
        $jwt = $login_json->{'token'};
        $CURL_GET = "curl -s -H \"Authorization:$jwt\" -X GET https://$master:1556/netbackup";
        $CURL_POST = "curl -s -H \"Authorization:$jwt\" -X POST https://$master:1556/netbackup";
}
sub add_role {
	my $cmd;
	my $diff_perm = "";
	# This adds version specific permission
	if ($version eq "8.1.2") {
		$diff_perm = ",{\\\"type\\\":\\\"permission\\\",\\\"id\\\":$nbu_permission_map{'Manage Appservers and Asset Groups'}}" ;
	} else {
		$diff_perm = ",{\\\"type\\\":\\\"permission\\\",\\\"id\\\":$nbu_permission_map{'View Storage'}}, {\\\"type\\\":\\\"permission\\\",\\\"id\\\":$nbu_permission_map{'Manage Application Servers and Asset Groups'}}, {\\\"type\\\":\\\"permission\\\",\\\"id\\\":$nbu_permission_map{'Overwrite Asset'}}" ;
	}

	# Command to create VRPServiceRole with all required permissions
	# This adds permission common for all NBU version
	$cmd = "$CURL_POST/rbac/roles -H \"Content-Type: application/vnd.netbackup+json;version=2.0\" -H \"accept: application/vnd.netbackup+json;version=2.0\"  -d \"{\\\"data\\\":{\\\"type\\\":\\\"role\\\",\\\"attributes\\\":{\\\"name\\\":\\\"VRPServiceRole\\\",\\\"description\\\":\\\"VRP role for asset recovery\\\"},\\\"relationships\\\":{\\\"permissions\\\":{\\\"data\\\":[{\\\"type\\\":\\\"permission\\\",\\\"id\\\":$nbu_permission_map{'View Hosts'}}, {\\\"type\\\":\\\"permission\\\",\\\"id\\\":$nbu_permission_map{'View Recovery Points'}},{\\\"type\\\":\\\"permission\\\",\\\"id\\\":$nbu_permission_map{'Recover/Restore'}},{\\\"type\\\":\\\"permission\\\",\\\"id\\\":$nbu_permission_map{'Instant Access'}},{\\\"type\\\":\\\"permission\\\",\\\"id\\\":$nbu_permission_map{'Download Files'}},{\\\"type\\\":\\\"permission\\\",\\\"id\\\":$nbu_permission_map{'Restore Files'}},{\\\"type\\\":\\\"permission\\\",\\\"id\\\":$nbu_permission_map{'Manage Hosts'}},{\\\"type\\\":\\\"permission\\\",\\\"id\\\":$nbu_permission_map{'Manage Policies'}},{\\\"type\\\":\\\"permission\\\",\\\"id\\\":$nbu_permission_map{'Manage SLPs'}},{\\\"type\\\":\\\"permission\\\",\\\"id\\\":$nbu_permission_map{'Manage Assets'}},{\\\"type\\\":\\\"permission\\\",\\\"id\\\":$nbu_permission_map{'Manage NetBackup'}}$diff_perm]}}}}\" --cacert $cacert_file";
	my $out_json = exec_cmd($cmd);
}

sub get_roles
{
        my $cmd = "$CURL_GET/rbac/roles -H \"accept: application/vnd.netbackup+json;version=2.0\" --cacert $cacert_file";
        my $roles_out = exec_cmd($cmd);
}

# This function helps to get a map of permission name to id
sub get_permissions
{
        my $cmd = "$CURL_GET/rbac/permissions -H \"accept: application/vnd.netbackup+json;version=2.0\" --cacert $cacert_file";
        my $perms_out = exec_cmd($cmd);
	if (!defined $perms_out || $perms_out eq "") {
                printf("\nError: Failed to get permission details\n");
                VRTS::Logger::log('error',"Erro: Failed to get permission details");
		exit(1);
        }

	foreach my $vrp_perm (@vrp_permission) {
		VRTS::Logger::log('debug',"Processing vrp permission [$vrp_perm]");
		foreach my $nbu_perm_map (@{$perms_out->{'data'}}) {
			if ($vrp_perm eq $nbu_perm_map->{'attributes'}->{'name'}) {
				$nbu_permission_map{$vrp_perm} = $nbu_perm_map->{'id'};
				VRTS::Logger::log('debug',"Found id as $nbu_perm_map->{'id'} for permission $vrp_perm");
			}
		}
	}
}

get_inputs();
get_cacert();
login();
get_permissions();
add_role();
printf("\nInfo: Role VRPServiceRole is successfully configured in NBU master. Create a user for on NBU master and assign this role to that user from NBU web ui. Use this user to configure NBU master in VRP\n");
VRTS::Logger::log('info',"Role 'VRPServiceRole' is successfully configured in NBU master. Create a user for on NBU master and assign this role to that user from NBU web ui. Use this user to configure NBU master in VRP");
unlink $cacert_file;
